//#########################################################################
// File: dosfind.c
//
// Demo wie man findfirst(), findnext() richtig benutzt.
// Hier quasi ein DIR oder LS Befehl mit Ausgabe ber RS232.
//
// Kopiere vom PC aus Dateien und Verzeichnisse auf deine CF/MMC/SD.
// Ihre Namen (DOS 8.3 und lange Namen) sollten alle per RS232 bertragen
// werden. Die Ausgabe erfolgt in der Reihenfolge wie sie im Verzeichnis
// abgelegt sind, also NICHT alphabetisch geordnet.
//
// 19200 Baud, 8 Bit, 1 Stopbit, kein Parity, kein Handshake
//
//#########################################################################
// Last change: 26.09.2006
//#########################################################################
// hk@holger-klabunde.de
// http://www.holger-klabunde.de/index.html
//#########################################################################
// Compiler: AVR-GCC 3.4.3
//#########################################################################
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdarg.h>
#include <string.h>
#include <stdlib.h>

#include "mydefs.h"
#include "protos.h"
#include "../dos.h"
#include "../serial.h"
#include "../printf.h"

//###################################################################################
/* Hauptprogramm */
int main(void)
//###################################################################################
{
 DDRC=0xFF;  	// set io-pins on PortC
 PORTC=0; 	//LCD-Datenbus

 DDRD=0xF2;  	// set io-pins on PortD
 PORTD=0x02; 	//LCD-Steuersignale siehe lcd.h

 DDRA=0; 	//Alles Eingnge
 PORTA=0xFF; 	//Eingnge mit Pullups setzen

#ifdef COMPACTFLASH_CARD
 DDRB=0xFF; 	//Alles Ausgnge
 PORTB=0xFF; 	
#endif

#ifdef MMC_CARD_SPI //SD_CARD_SPI too !
 MMC_IO_Init();
#endif

 SFIOR&=!(1<<PUD); 	//Pullups Enable
// SFIOR=0; 	//Pullups Enable

 ser_init();

 sei();              //enable interrupts

 puts("FAT DOS-Find-Test\n");

 if(GetDriveInformation()!=F_OK) // get drive parameters
  {
   puts("Flash meldet sich nicht !");
   while(1);
  }
  
 RED_ON();
 GREEN_OFF();

 if(Findfirst()!=0) //Finde erste Datei/Verzeichnis im aktuellen Verzeichnis
  {
   do
    {
     printf("%s   ",ffblk.ff_name); //Der 8.3 DOS Name
     if(ffblk.ff_attr==ATTR_FILE) printf("% 9lu\n",ffblk.ff_fsize);
     if(ffblk.ff_attr==ATTR_DIRECTORY) printf("<DIR>\n");

     //Hat die Datei einen langen Dateinamen ?
     if(ffblk.ff_longname[0]!=0) printf("%s\n",ffblk.ff_longname);

    }while(Findnext()!=0); //Finde nchste Datei/Verzeichnis im aktuellen Verzeichnis
  }
 else
  {
   puts("No file/dir found !\n");
  }

 GREEN_ON();
 RED_OFF();
 cli(); // Disable interrupts
 
 for(;;) // loop forever
  {
  }
}

