//#########################################################################
// File: main.c
//
// Erster Versuch MP3's mit VS1001K und meinen FAT Routinen abzuspielen.
// Ich wollte wissen ob fread() schnell genug ist um MP3's abzuspielen.
// Ein FAT Buffer ist nicht erforderlich !
//
// Mit 4MHz konnten Dateien mit 224kbps abgespielt werden.
// Ab 6MHz knnen 320kbps ohne Probleme abgespielt werden.
// Und das mit Ausgabe auf LCD UND RS232 !
//
// Dieses Programm wurde bisher nur mit MMC/SD und ATMega32 getestet.
// RAM Bedarf ca. 1.5kB. Code ca. 11kB.
//
// Das hier soll kein komfortabler MP3 Player werden ! Davon gibt es genug.
// Es ist nur ein ### TESTPROGRAMM ###
//
// Im Root Verzeichnis mu ein Verzeichnis MP3 existieren.
// Das Testprogramm spielt alle MP3 Dateien im MP3-Verzeichnis ab.
// Das wird solange wiederholt bis STOP gedrckt wird.
// Also ideal als endlose Hintergrundmusik, z.B. fr die Telefonanlage,
// weil es keine beweglichen Teile gibt die verschleien knnen.
// Es drfen auch andere Dateien auf dem Datentrger liegen. Sie werden
// ganz einfach ignoriert.
//
// Auf RS232 wird der DOS 8.3 oder der lange Dateiname des aktuellen
// Songs ausgegeben. Parameter 38400 Baud,8 Bit,no Parity,1 Stopbit
// Da ich ein 40x2 LCD benutzt habe wurde die Ausgabe von langen Dateinamen
// in find_x.h auf 40 Zeichen gekappt. Also kein scrolling des Titels.
// Die LCD Ausgabe funktioniert hier nur mit dem !!! ATMega32 !!!
// Und nur mit 40x2 Displays. Nicht grer, nicht kleiner.
//
// Die Dateien werden in der Reihenfolge abgespielt wie sie im Verzeichnis
// liegen. Das mu NICHT heien das sie in alphabetischer Reihenfolge
// abgespielt werden !
//
// Eine Tastatur mit 8 Tasten hngt an PORTA. Fr CompactFlash mu die anders
// angeschlossen werden ! Tastenbelegung siehe mydefs.h.
//
// Fr bestes Str/Nutzverhltnis sollte man bei Line-In Eingngen
// die Lautstrke vom Line-In Eingang erst runterdrehen und den Player
// voll aufdrehen. Dann die gewnschte Lautstrke mit dem Regler vom
// Line-In Eingang einstellen. Macht man es umgekehrt kann man evtl.
// leise Strgerusche aus dem Player hren. Mit einem Kopfhrer konnte
// ich keine Strgerusche aus dem Player wahrnehmen. Das erste Layout
// scheint ganz gut gelungen zu sein :)
//
// Beim einschalten des Players gibt es einen unangenehmen Knacks.
// Der ist immer zu hren wenn der Hardware Reset durchgefhrt wird.
// Beim Software Reset zwischen den Songs gibt es keine Knackser.
//
//#########################################################################
// Tastenfunktionen:
// Play, Pause, Stop, Replay, Volume up, Volume down, Next song, Prev song
// Funktion ungefhr so wie bei einem CD-Player.
// Volume up/down, Next song, Prev song funktioniert nur wenn ein Song
// gespielt wird !
//
// Todo: Mit Next kann man vom letzten Song auf den ersten Song springen.
// Man kann aber nicht wenn der erste Song gespielt wird mit Prev auf den
// letzten Song springen ! Also kein vollstndiger Kreis fr Next,Prev.
//
//#########################################################################
// Last change: 03.09.2006
//#########################################################################
// hk@holger-klabunde.de
// http://www.holger-klabunde.de/index.html
//#########################################################################
// Compiler: AVR-GCC 3.4.5
//#########################################################################
#include <avr/io.h>
#include <avr/interrupt.h>

#include <stdarg.h>
#include <string.h>
#include <stdlib.h>

#include "mydefs.h"
#include "protos.h"
#include "../dos.h"
#include "../serial.h"
#include "../printf.h"
#include "../lcd.h"

#include "vs1001.h"
#include "rc5.h"
#include "buttons.h"
#include "display.h"
#include "playmp3.h"

volatile unsigned char play_status;
unsigned char volume;

//###################################################################################
/* Hauptprogramm */
int main(void)
//###################################################################################
{
 unsigned char result;
 
 Delay1ms(200); //Wait for Power Up of MMC/SD/CF and VS1001

//1 Ausgang
//0 Eingang

 DDRC=0xFF;  	// set io-pins on PortC
 PORTC=0; 	//LCD-Datenbus

 DDRD=0xF2;  	// set io-pins on PortD
 PORTD=0x02; 	//LCD-Steuersignale siehe lcd.h

#ifdef COMPACTFLASH_CARD
#if defined (__AVR_ATmega32__) || defined (__AVR_ATmega323__)
#  error "CompactFlash does not work with this processor type in main.c"
#endif
#endif //#ifdef COMPACTFLASH_CARD

#ifdef MMC_CARD_SPI //SD_CARD_SPI too !
 MMC_IO_Init();
#endif

 VS1001_IO_Init();

 SFIOR&=!(1<<PUD); 	//Pullups Enable

 ser_init(); //init serial port

 RemoteInit();
 ButtonsInit();
 DisplayInit();
 
 sei();              //enable interrupts

 printf("Try to read FlashDrive Configuration\n");

 if(GetDriveInformation()!=F_OK) // get drive parameters
  {
   printf("No Flash found !");
   RED_OFF();
   GREEN_ON();
   while(1);
  }

 LCDCls();
  
 VS1001_Reset(HARD_RESET);
 // Switching the hardware reset line of the VS1001 results in a loud ugly blop.

 RED_ON();
 GREEN_OFF();

 // Todo:
 // Get last values from eeprom for
 // Volume, last played title, last player status (stop/play)
 // Bass/trebble enhancer on/off
 // A balance function !
 
 // Volume 0 is highest volume
 // Volume 254 is lowest volume
 // Volume 255 is power down VS1001

 volume=0; //play it loud ;)
 VS1001_SetVolume(volume,volume);

 VS1001_SetMode(VS_SM_BASS); //Bass/treble enhancer. Good for small headphones.
// VS1001_SetMode(VS_SM_DIFF); //Virtual surround stereo test ;)
 
// Don't do this WITHOUT a FAT buffer !
// printf("Drive Size %lukB\n",drivesize());
// printf("Drive Free %lukB\n",drivefree());
// printf("Drive Used %lukB\n",driveused());

// RED_OFF();
// GREEN_ON();
 play_status=PLAYER_PLAY; //auto play on power on. remove if you don't like it
 result=0;

 chdir("mp3"); //change to MP3 directory !

// Player buttons are read in TCNT1 interrupt routine !
// Remote control signals are read in INT0 interrupt routine !
 
 for(;;) // loop forever until power down
  {
   CheckRemote();
   //Player buttons are read in interrupt routine !
   
   if(findfirst()!=0 && play_status==PLAYER_PLAY) //find FIRST file in directory
    {
     do
      {
       if(ffblk.ff_attr==ATTR_FILE && play_status==PLAYER_PLAY) //did we find a file ?
        {

         //is this a MP3 file ? check the DOS 8.3 extension to .MP3
         if(ffblk.ff_name[9]=='M' && ffblk.ff_name[10]=='P' && ffblk.ff_name[11]=='3')
          {
           LCDCls();
           
           //do we have a long filename ?
           if(ffblk.ff_longname[0]!=0) printf("%s\n",ffblk.ff_longname); //print long name
           else printf("%s\n",ffblk.ff_name);    //print 8.3 DOS name
//           printf("% 9lu\n",ffblk.ff_fsize); //print filesize 

           PlayMp3(ffblk.ff_name);           //play the song

          }//if(ffblk.ff_name[9]=='M' && ...
        }//if(ffblk.ff_attr==ATTR_FILE && play_status ...

       if(play_status==PLAYER_PLAY) result=findnext(); //find the next song
 
      }while(result!=0 && play_status!=PLAYER_STOP);

      //At this point last song is played. So go to the first file and play all 
      //songs again until power is switched off.
    }
  }//for(;;)
}
