//######################################################################
// File: playmp3.c
//
//######################################################################
// Last change: 01.09.2006
//#########################################################################
// hk@holger-klabunde.de
// http://www.holger-klabunde.de/index.html
//#########################################################################
// Compiler: AVR-GCC 3.4.5
//#########################################################################
#include <avr/io.h>
#include <avr/interrupt.h>

#include "mydefs.h"
#include "../dos.h"

#include "playmp3.h"
#include "buttons.h"
#include "vs1001.h"
#include "display.h"
#include "rc5.h"

//###################################################################################
// MP3 playing routine with control buttons, remote checking and display output
// fread() was not fast enough to play MP3's with 256/320kbps at 4MHz !
// fread() does it at least with 6MHz.
// So this is a modified routine ReadFileRaw(). 
int PlayMp3(char *name)
//###################################################################################
{
 unsigned long tmpsector;

#ifdef USE_FAT32
 unsigned long tmp;
#else
 unsigned int tmp;
#endif

 unsigned char *p;
 unsigned char secloop;
 unsigned int i;
 unsigned char j,result;

 if(FindName(name)==FULL_MATCH) //does file exist ?
  {
   tmp=FileFirstCluster;
   result=F_OK;
  
   //##########################
   // Beginning of play loop  
   //##########################

   while(tmp < endofclusterchain) // Read all clusters of file.
    {

     if(play_status==PLAYER_PLAY) // Read nothing if playing is paused !
      {

       tmpsector=GetFirstSectorOfCluster(tmp);

       // Read all sectors of the current file cluster.
       // After one cluster is send to the VS1001, check player buttons and remote status.
       for(secloop=0; secloop<secPerCluster; secloop++)
        {
         ReadSector(tmpsector,iob);

         // Read data from iob now
         for(i=0; i<BYTE_PER_SEC; i+=32)
          {
           p=&iob[i];          //pointer to io buffer

           // Fill VS1001 buffer until DREQ is set to 0 or end of file reached
           while(VS1001_BUSY); // Wait for VS1001 if busy.

           if(FileSize<32)
            {
             for(j=(unsigned char)FileSize; j<32; j++) *&p[j]=0; // Set unused bytes to zero 
             result=F_ERROR; // End of file reached if FileSize < 32
            } 
         
           if(FileSize>0) VS1001_WriteData32(p); //give VS1001 something to play
         
           FileSize-=32; // count down bytes read

           if(result==F_ERROR) break;
          } //for(i=0; i<BYTE_PER_SEC; i+=32)

         tmpsector++; // next sector of cluster

         if(result==F_ERROR) break;
        }// for(secloop=0; secloop<secPerCluster; secloop++)

      }//if(play_status==PLAYER_PLAY)  

     //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     //VS1001 buffer is full here, so we have some time to do other things
     //like testing input keys, remote control or update display informations.
     //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

#ifdef USE_REMOTE
     CheckRemote();
#endif

     CheckButtonsInLoop();

     UpdateDisplay();

     if(play_status==PLAYER_PLAY) // Do nothing if playing is paused
      {
       tmp=GetNextClusterNumber(tmp);
      }//if(play_status==PLAYER_PLAY)  

     // Read file to the end, or STOP, or NEXT, or PREV, or REPLAY
     if(result==F_ERROR) break;
     if(play_status!=PLAYER_PLAY && play_status!=PLAYER_PAUSE) break;
     
    }// while(tmp < endofclusterchain)

   //#####################
   // End of playing loop  
   //#####################

   VS1001_Reset(SOFT_RESET); // Very important ! soft reset vs1001k after each track

   CheckButtonsAfterLoop();

   return 0; //no errors
  }//if(FindName(name)==FULL_MATCH)

 return 1; //error
}
