//######################################################################
// File: playmp3.c
//
//######################################################################
// Last change: 01.09.2006
//#########################################################################
// hk@holger-klabunde.de
// http://www.holger-klabunde.de/index.html
//#########################################################################
// Compiler: AVR-GCC 3.4.5
//#########################################################################
#include <avr/io.h>
#include <avr/interrupt.h>

#include "mydefs.h"
#include "../dos.h"

#include "playmp3.h"
#include "buttons.h"
#include "rc5.h"
#include "vs1001.h"
#include "display.h"

//###################################################################################
// MP3 playing routine with control buttons checking and display output
int PlayMp3(char *name)
//###################################################################################
{
 unsigned char result, buf[PLAY_BUFFER_SIZE], packets;
 unsigned int i, numbytes;

 packets=PLAYER_MAX_PACKETS; //Allows you to break playing if player hangs or
                             //does not play correctly. For example if you want to play
                             //320kbps files with CPU clock of 4.00MHz.
 
 result=fopen(name,F_READ);
 if(result==F_OK)
  {
   //#####################
   //begin of playing loop  
   //#####################
   do
    {

     if(play_status==PLAYER_PLAY) //read nothing if playing is paused
      {

       //Fill VS1001 buffer until DREQ is set to 0 or end of file reached
       while((VS1001_READY_FOR_RECEIVE) && result==F_OK)
        {

         //read up to PLAY_BUFFER_SIZE Bytes from cf/mmc/sd for the VS1001 buffer
         numbytes=fread(buf,PLAY_BUFFER_SIZE);
         if(numbytes!=PLAY_BUFFER_SIZE)
          {
           for(i=numbytes; i<PLAY_BUFFER_SIZE; i++) buf[i]=0;
           result=F_ERROR; //end of file reached ?
          } 
         
         if(numbytes>0) VS1001_WriteData32(buf); //give VS1001 something to play
         
         packets--; //count down PLAYER_MAX_PACKETS counter
         if(packets==0)
          {
           packets=PLAYER_MAX_PACKETS;
           break; //force breaking the loop
          }
        }

      }//if(play_status==PLAYER_PLAY)  

     //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
     //VS1001 buffer is full here, so we have some time to do other things
     //like testing input keys, remote control or update display informations.
     //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

     CheckRemote();
     CheckButtonsInLoop();

     UpdateDisplay();

    }while(result==F_OK && ( play_status==PLAYER_PLAY || play_status==PLAYER_PAUSE));
   //read file to the end or STOP or NEXT or PREV or REPLAY

   //#####################
   //end of playing loop  
   //#####################

   VS1001_Reset(SOFT_RESET); //very important ! soft reset vs1001k after each track

   CheckButtonsAfterLoop();

   fclose(); //close this MP3 file
   return 0; //no errors
  }
 else return 1; //error
// else printf("File is not open !\n");

}
